package com.jobs.controller;

import com.github.benmanes.caffeine.cache.Cache;
import com.jobs.pojo.Employee;
import com.jobs.pojo.Result;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.Random;

@RequestMapping("/emp")
@RestController
public class EmployeeController {

    @Autowired
    private Cache<String, Employee> employeeCache;

    @PostMapping("/add")
    public Result<Boolean> addEmployee(Employee emp) {
        employeeCache.put(emp.getName(), emp);
        return Result.success(true);
    }

    @PostMapping("/update")
    public Result<Boolean> updateEmployee(Employee emp) {
        employeeCache.put(emp.getName(), emp);
        return Result.success(true);
    }

    @GetMapping("/get/{name}")
    public Result<Employee> getEmployee(@PathVariable("name") String name) {
        //如果能够取到，则返回获取的内容
        //如果取不到，则执行方法（比如去数据库中获取），然后存储到缓存中
        Employee employee = employeeCache.get(name,
                key -> new Employee(key, new Random().nextInt(50)));
        return Result.success(employee);
    }

    @DeleteMapping("/delete")
    public Result<Boolean> deleteEmployee(String name) {
        employeeCache.invalidate(name);
        return Result.success(true);
    }

}
