package com.jobs;

import com.github.benmanes.caffeine.cache.Cache;
import com.github.benmanes.caffeine.cache.Caffeine;
import org.junit.jupiter.api.Test;
import org.springframework.boot.test.context.SpringBootTest;

import java.time.Duration;

@SpringBootTest
public class CaffeineTest {

    @Test
    void test1() {
        // 构建 cache 对象，由于是泛型，因此可以声明出你想要的类型
        Cache<String, String> cache = Caffeine.newBuilder().build();

        // 存数据 或 更新数据
        cache.put("user", "侯胖胖");

        // 取数据
        String user = cache.getIfPresent("user");
        System.out.println("user = " + user);

        // 取数据，如果未取到，可以执行方法，比如查询数据库，然后把结果自动存储到缓存中
        String fatUser = cache.get("fatUser", key -> {
            return "任肥肥";
        });
        System.out.println("fatUser = " + fatUser);
    }

    //设置最大容量，超过容量的话，清除之前添加的缓存
    @Test
    void test2() throws InterruptedException {
        // 创建缓存对象
        Cache<String, String> cache = Caffeine.newBuilder()
                // 设置缓存最大容量上限为 1
                .maximumSize(1).build();

        cache.put("user1", "侯胖胖");
        cache.put("user2", "任肥肥");
        cache.put("user3", "乔豆豆");

        // 延迟10ms，给清理线程一点时间
        Thread.sleep(10L);

        // 获取数据
        System.out.println("user1: " + cache.getIfPresent("user1"));
        System.out.println("user2: " + cache.getIfPresent("user2"));
        System.out.println("user3: " + cache.getIfPresent("user3"));
    }

    //设置缓存的有效期，超时后，将获取不到
    @Test
    void test3() throws InterruptedException {
        // 创建缓存对象
        Cache<String, String> cache = Caffeine.newBuilder()
                // 设置缓存有效期为 1 秒
                .expireAfterWrite(Duration.ofSeconds(1))
                .build();
        // 存数据
        cache.put("user", "乔豆豆");
        // 获取数据
        System.out.println("user: " + cache.getIfPresent("user"));
        // 休眠一会儿
        Thread.sleep(1300L);
        // 再获取数据
        System.out.println("user: " + cache.getIfPresent("user"));
    }

}
